% Copyright (C) 2017, 2018 Rune Kjerrumgaard Jensen,  
% Jesper Kr Larsen, and  Kasper Lindgren Lassen
%
% Permission is hereby granted, free of charge, to any person obtaining
% a copy of this software and associated documentation files (the
% "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to
% permit persons to whom the Software is furnished to do so, subject to
% the following conditions:
%
% The above copyright notice and this permission notice shall be
% included in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL AUTHORS BE LIABLE FOR ANY
% CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
% SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Except as contained in this notice, the name of the authors shall
% not be used in advertising or otherwise to promote the sale, use or
% other dealings in this Software without prior written authorization
% from the authors.

function f = FricFac(D, roughness, rho, Q, viscosity, A)
%% Calcuation of the Darcy friction factor.
% Reynolds number                                                           [-]
Re = rho*D*abs(Q)/(viscosity*A);

if Re==0
    % Darcy friction factor for zero flow                                   [-]
    f = 1;
elseif Re>0 && Re<2100
    %% Laminar flow
    % Darcy friction factor for laminar flow                                [-]
    f = 64/Re;
else
    %% Colebrook-White equation
    % The friction factor, f, is solved iteratively with ff being the initial
    % value, and the accepted error being 1E-12. err is the error compared
    % with the allow error, and the friction factor is only accepted when
    % err becomes lower than 1E-12.
    % Initializing ff                                                       [-]
    ff = 10;
    % Setting lower error limit                                             [-]
    err = 0.0001;
    % Initializing Darcy frition factor                                     [-]
    f_old = 0;
    
    while err > 1E-12
        % Darcy friction factor for turbulent flow                          [-]
        f = 1/ff^2;
        % Result from Colebrook equation                                    [-]
        ff = -2*log10(roughness/(3.7*D)+2.51/(Re*sqrt(f)));
        % Error between iterations                                          [-]
        err = abs(f - f_old);
        % Darcy friction factor for turbulent flow                          [-]
        f_old = f;
    end
end
end