% Copyright (C) 2017, 2018 Rune Kjerrumgaard Jensen,  
% Jesper Kr Larsen, and  Kasper Lindgren Lassen
%
% Permission is hereby granted, free of charge, to any person obtaining
% a copy of this software and associated documentation files (the
% "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to
% permit persons to whom the Software is furnished to do so, subject to
% the following conditions:
%
% The above copyright notice and this permission notice shall be
% included in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL AUTHORS BE LIABLE FOR ANY
% CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
% SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Except as contained in this notice, the name of the authors shall
% not be used in advertising or otherwise to promote the sale, use or
% other dealings in this Software without prior written authorization
% from the authors.

function [J] = FrictionTerm(Friction_Type, D, roughness, rho, Re_0, Q_0,...
    Q_point, Q, viscosity, A, i, j, dx, dt, Charact_Line, g, a, f_pre,...
    W, dQ, n_t)
% The friction term is comprised of three parts, steady state friction ,J_s
% (only used by "Prescribed_Steady_State_Friction" and
% "Steady_State_Friction"), quasi-steady friction, J_q, and unsteady
% friction, J_u (for the unsteady friction models). Theses three are
% summarized after the calculation of each part.
switch Friction_Type
    case 'Prescribed_Steady_State_Friction'
        % Resistance coefficient                                            [s^2/m^5]
        R = f_pre*dx/(2*g*D*A^2);
        J_s = R*Q_point*abs(Q_point);
        J_q = 0;
        J_u = 0;
    case 'Steady_State_Friction'
        % Darcy friction factor, determined for either laminar flow or via
        % the Colebrook-White equation for turbulent flow                   [-]
        f_s = FricFac(D, roughness, rho, Q_0, viscosity, A);
        % Resistance coefficient                                            [s^2/m^5]
        R = f_s*dx/(2*g*D*A^2);
        J_s = R*Q_point*abs(Q_point);
        J_q = 0;
        J_u = 0;
    case 'Quasi_Steady_Friction'
        % Darcy friction factor, determined for either laminar flow or via
        % the Colebrook-White equation for turbulent flow                   [-]
        f_q = FricFac(D, roughness, rho, Q_point, viscosity, A);
        % Resistance coefficient                                            [s^2/m^5]
        R = f_q*dx/(2*g*D*A^2);
        J_s = 0;
        J_q = R*Q_point*abs(Q_point);
        J_u = 0;
    case 'Unsteady_Friction_Brunone'
        % Darcy friction factor, determined for either laminar flow or via
        % the Colebrook-White equation for turbulent flow                   [-]
        f_q = FricFac(D, roughness, rho, Q_point, viscosity, A);
        % Resistance coefficient                                            [s^2/m^5]
        R = f_q*dx/(2*g*D*A^2);
        J_s = 0;
        J_q = R*Q_point*abs(Q_point);
        switch Charact_Line
            case 'Plus'
                J_u = BrunoneFricp(Q, Re_0, D, j, i, a, dt, dx,g,A);
            case 'Minus'
                J_u = BrunoneFricm(Q, Re_0, D, j, i, a, dt, dx,g,A);
        end
    case 'Unsteady_Friction_Zielke'
        % Darcy friction factor, determined for either laminar flow or via
        % the Colebrook-White equation for turbulent flow                   [-]
        f_q = FricFac(D, roughness, rho, Q_point, viscosity, A);
        % Resistance coefficient                                            [s^2/m^5]
        R = f_q*dx/(2*g*D*A^2);
        J_s = 0;
        J_q = R*Q_point*abs(Q_point);
        switch Charact_Line
            case 'Plus'
                J_u = CBFricp(dt, j, i, D, rho, viscosity, g,A,a,W,dQ,n_t);
            case 'Minus'
                J_u = CBFricm(dt, j, i, D, rho, viscosity, g,A,a,W,dQ,n_t);
        end
    case 'Unsteady_Friction_VardyBrown'
        % Darcy friction factor, determined for either laminar flow or via
        % the Colebrook-White equation for turbulent flow                   [-]
        f_q = FricFac(D, roughness, rho, Q_point, viscosity, A);
        % Resistance coefficient                                            [s^2/m^5]
        R = f_q*dx/(2*g*D*A^2);
        J_s = 0;
        J_q = R*Q_point*abs(Q_point);
        switch Charact_Line
            case 'Plus'
                J_u = CBFricp(dt, j, i, D, rho, viscosity, g,A,a,W,dQ,n_t);
            case 'Minus'
                J_u = CBFricm(dt, j, i, D, rho, viscosity, g,A,a,W,dQ,n_t);
        end
    case 'Unsteady_Friction_Zarzycki'
        % Darcy friction factor, determined for either laminar flow or via
        % the Colebrook-White equation for turbulent flow                   [-]
        f_q = FricFac(D, roughness, rho, Q_point, viscosity, A);
        % Resistance coefficient                                            [s^2/m^5]
        R = f_q*dx/(2*g*D*A^2);
        J_s = 0;
        J_q = R*Q_point*abs(Q_point);
        switch Charact_Line
            case 'Plus'
                J_u = CBFricp(dt, j, i, D, rho, viscosity, g,A,a,W,dQ,n_t);
            case 'Minus'
                J_u = CBFricm(dt, j, i, D, rho, viscosity, g,A,a,W,dQ,n_t);
        end
end
% Friction term                                                             [m]
J = J_s + J_q + J_u;
end
