%% Choose Solver, Boundary Conditions, Wave Speed method and Friction Type
% Choose solver:
% 1)    1D_SinglePhase
% 2)    1D_TwoPhase_DVCM
% 3)    1D_TwoPhase_DGCM
Solver = '1D_TwoPhase_DVCM';

% Choose upstream boundary condition:
% 1)    Reservoir
Upstream_boundary = 'Reservoir';

% Choose downstream boundary condition:
% 1)    Valve_Instantaneous_Closure
% 2)    Valve_Transient_Closure
Downstream_boundary = 'Valve_Transient_Closure';

% Choose wave speed method: already known or need calculation:
% 1)    WaveSpeed_Known
% 2)    WaveSpeed_Calculate
WaveSpeed_Type = 'WaveSpeed_Known';

% Choose friction type:
% 1)    Prescribed_Steady_State_Friction (insert value in f_pre)
% 2)    Steady_State_Friction
% 3)    Quasi_Steady_Friction
% 4)    Unsteady_Friction_Brunone
% 5)    Unsteady_Friction_Zielke
% 6)    Unsteady_Friction_VardyBrown
% 7)    Unsteady_Friction_Zarzycki
Friction_Type = 'Quasi_Steady_Friction';

%% Input Data
% Mesh
Reaches = 24;                           % Number of divisions of the pipe.
Oscillations = 11;                      % One oscillation is four times the traveling time of the pressure wave.

% Universal Constants
g = 9.8;                                % Gravitational acceleration    [m/s^2]

% Pipe Dimensions and Parameters
L = 37.23;                              % Length of pipe                [m]
D = 22.1e-3;                            % Diameter of pipe              [m]
A = pi * D^2/4;                         % Cross sectional area of pipe  [m^2]
e = 1.63e-3;                            % Thickness of pipe             [m]
E = 120E9;                              % Young's modulus               [Pa]
roughness  = 0.0015E-3;                 % Absolute roughness            [m]
nu_p = 0.35;                            % Poisson's ratio               [-]
theta = atand(0.0545);                  % Angle of inclination          [-]

% Fluid Properties
rho = 998.2;                            % Density of water              [kg/m^3]
K   = 2.2E9;                            % Bulk modulus of water         [Pa]
viscosity = 1.002E-3;                   % Dynamic viscosity             [kg/m*s]

switch Solver
    case '1D_TwoPhase_DVCM'
        H_vap = 0.10793;                 % Vapour pressure head              [m]
        H_b = 101325/(rho*g);           % Barometric pressure head          [m]
        H_v = H_vap - H_b;
    case '1D_TwoPhase_DGCM'
        H_sat = 0.10793;                 % Vapour pressure head              [m]
        H_b = 101325/(rho*g);           % Barometric pressure head          [m]
        H_v = H_sat - H_b;
        alpha_0 = 1e-7;                   % Void fraction at reference pressure [-]
end

% Weighting factor for DVCM and DGCM
psi = 0.55;                             % [-]

% Flow Inputs
u_0 = 1.40;                                % Initial flow velocity         [m/s]
Q_0 = u_0*A;                            % Initial volumetric flow rate  [m^3/s]
Re_0 = rho*u_0*D/viscosity;             % Initial Reynolds number       [-]

% Upstream Reservoir / Initial head
H_r = 22;                           % Height of reservoir           [m]

% Valve 
t_c = 0.009;                          % Closing time of valve         [s]
switch Downstream_boundary
    case 'Valve_Instantaneous_Closure'
        m = 0;                          % Valve closure coefficient  [-]
    case 'Valve_Transient_Closure'
        m = 5;                          % Valve closure coefficient  [-]
end

% Wave speed - pure liquid
switch WaveSpeed_Type
    case 'WaveSpeed_Calculate'
        a = WaveSpeed(e,D,K,rho,E,nu_p);  % Speed of the pressure wave    [m/s]
    case 'WaveSpeed_Known'
        a = 1319;                       % Speed of the pressure wave    [m/s]
end

% Prescribed steady state friction coefficient (do not remove or hide, because the script might give an error even though it do not use it)
f_pre = 0;                           % Prescribed steady state fricion coefficient [-]
