%% Choose Solver, Boundary Conditions, Wave Speed method and Friction Type
% Choose solver:
% 1)    1D_SinglePhase
% 2)    1D_TwoPhase_DVCM
% 3)    1D_TwoPhase_DGCM
Solver = '1D_TwoPhase_DGCM';

% Choose upstream boundary condition:
% 1)    Reservoir
Upstream_boundary = 'Reservoir';

% Choose downstream boundary condition:
% 1)    Valve_Instantaneous_Closure
% 2)    Valve_Transient_Closure
Downstream_boundary = 'Valve_Transient_Closure';

% Choose wave speed method: already known or need calculation:
% 1)    WaveSpeed_Known
% 2)    WaveSpeed_Calculate
WaveSpeed_Type = 'WaveSpeed_Calculate';

% Choose friction type:
% 1)    Prescribed_Steady_State_Friction (insert value in f_pre)
% 2)    Steady_State_Friction
% 3)    Quasi_Steady_Friction
% 4)    Unsteady_Friction_Brunone
% 5)    Unsteady_Friction_Zielke
% 6)    Unsteady_Friction_VardyBrown
% 7)    Unsteady_Friction_Zarzycki
Friction_Type = 'Unsteady_Friction_VardyBrown';

%% Mesh
% Number of divisions of the pipe                                           [-]
Reaches = 48;
% One oscillation is four times the traveling time of the pressure wave     [-]
Oscillations = 20;

%% Universal Constants
% Gravitational acceleration                                                [m/s^2]
g = 9.8;

%% Pipe Dimensions and Parameters
% Length of pipe                                                            [m]
L = 15.22;
% Diameter of pipe                                                          [m]
D = 0.02;
% Cross sectional area of pipe                                              [m^2]
A = pi * D^2/4;
% Thickness of pipe                                                         [m]
e = 0.001;
% Young's modulus                                                           [Pa]
E = 120E9;
% Absolute roughness                                                        [m]
roughness  = 0.0015E-3;
% Poisson's ratio                                                           [-]
nu_p = 0.35;
% Angle of inclination                                                      [deg]
theta = 0;

%% Fluid Properties
% Density of water                                                          [kg/m^3]
rho = 998.2;
% Bulk modulus of water                                                     [Pa]
K   = 2.2E9;
% Dynamic viscosity                                                         [kg/m*s]
viscosity = 1.002E-3;

switch Solver
    case '1D_TwoPhase_DVCM'
        % Vapour pressure in piezometric head                               [m]
        H_vap = 0.10793;
        % Barometric pressure head                                          [m]
        H_b = 101325/(rho*g);
        % Vapour pressure in gauge piezometric head                         [m]
        H_v = H_vap - H_b;
    case '1D_TwoPhase_DGCM'
        % Saturation pressure in piezometric head                           [m]
        H_sat = 0.10793;
        % Barometric pressure head                                          [m]
        H_b = 101325/(rho*g);
        % Saturation pressure in gauge piezometric head                     [m]
        H_v = H_sat - H_b;
        % Void fraction at reference pressure                               [-]
        alpha_0 = 1e-7;
end

%% Weighting factor for DVCM and DGCM
% Weighting factor                                                          [-]
psi = 0.55;

%% Flow Inputs
% Initial flow velocity                                                     [m/s]
u_0 = 0.156e-3/A;
% Initial volumetric flow rate                                              [m^3/s]
Q_0 = u_0*A;
% Initial Reynolds number                                                   [-]
Re_0 = rho*u_0*D/viscosity;

%% Upstream reservoir / Initial head
% Height/pressure of the reservoir                                          [m]
H_r = 46;

%% Downstream valve 
% Closing time of valve                                                     [s]
t_c = 18/1000;

switch Downstream_boundary
    case 'Valve_Instantaneous_Closure'
        % Valve closure coefficient                                         [-]
        m = 0;
    case 'Valve_Transient_Closure'
        % Valve closure coefficient                                         [-]
        m = 5;
end

%% Wave speed - pure liquid
switch WaveSpeed_Type
    case 'WaveSpeed_Calculate'
        % Speed of the pressure wave                                        [m/s]
        a = WaveSpeed(e,D,K,rho,E,nu_p);
    case 'WaveSpeed_Known'
        % Speed of the pressure wave                                        [m/s]
        a = 1200;
end

%% Prescribed steady state friction coefficient (do not remove or hide)
% Prescribed steady state fricion coefficient [-]
f_pre = 0;
