% Copyright (C) 2017, 2018 Rune Kjerrumgaard Jensen,  
% Jesper Kr Larsen, and  Kasper Lindgren Lassen
%
% Permission is hereby granted, free of charge, to any person obtaining
% a copy of this software and associated documentation files (the
% "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to
% permit persons to whom the Software is furnished to do so, subject to
% the following conditions:
%
% The above copyright notice and this permission notice shall be
% included in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL AUTHORS BE LIABLE FOR ANY
% CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
% SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Except as contained in this notice, the name of the authors shall
% not be used in advertising or otherwise to promote the sale, use or
% other dealings in this Software without prior written authorization
% from the authors.

function [Q_u_P, Q_P, H_P, V_g_P] = InteriorNodes_DGCM(a, g, A, rho, D,...
    viscosity, roughness, dx, theta, Q_A, H_A, Q_u_B, H_B, Q, Q_u, Re_0,...
    i, j, dt, Friction_Type, Q_0, f_pre, W, dQ, n_t, H_0, alpha_0,...
    V_total, V_g_P0, Q_P0, Q_u_P0, psi, z_P, H_v)
%% Interior nodes
% Pipe constant                                                             [s/m^2]
B = PipeConst(a, g, A);

% Positive characteristics equation                                         [m]
C_p = Characteristic_Plus(a, g, A, D, dx, roughness, rho, viscosity,...
    theta,  Q_A, H_A, Q, Re_0, i, j, dt, Friction_Type, Q_0, f_pre, W,...
    dQ, n_t);

% Negative characteristics equation                                         [m]
C_m = Characteristic_Minus(a, g, A, D, dx, roughness, rho, viscosity,...
    theta, Q_u_B, H_B, Q_u, Re_0, i, j, dt, Friction_Type, Q_0, f_pre, W,...
    dQ, n_t);

%% Calculation of head - Fluid Transients in Systems
% Pressure at steady state                                                  [Pa]
P_0 = rho*g*(H_0 - z_P - H_v);
% Constant                                                                  [m^4]
C_3 = P_0*alpha_0*V_total/(rho*g);
% Constant                                                                  [-]
B_2 = 0.5/2;
% Constant                                                                  [m^2]
C_4 = B_2*B*C_3/(psi*dt);
% Constant                                                                  [m^3/s]
B_v = (V_g_P0/(2*dt) + (1 - psi)*(Q_P0 - Q_u_P0))/psi;

if B_v <= 0
    % Constant                                                              [m^3/s]
    B_v = 0;
end

% Constant                                                                  [m/s]
B_1 = -B_2*(C_m + C_p) + B_2*B*B_v + (z_P + H_v)/2;

if B_1 == 0
    % Piezometric head                                                      [m]
    H_P = sqrt(C_4) + z_P + H_v;
else
    % Constant                                                              [-]
    B_B = C_4/B_1^2;
    
    if B_1 < 0 && B_B > 0.001
        % Piezometric head                                                  [m]
        H_P = -B_1*(1 + sqrt(1 + B_B)) + z_P + H_v;
    elseif B_1 > 0 && B_B > 0.001
        % Piezometric head                                                  [m]
        H_P = -B_1*(1 - sqrt(1 + B_B)) + z_P + H_v;
    elseif B_1 < 0 && B_B < 0.001
        % Piezometric head                                                  [m]
        H_P = -2*B_1 - C_4/(2*B_1) + z_P + H_v;
    elseif B_1 > 0 && B_B < 0.001
        % Piezometric head                                                  [m]
        H_P = C_4/(2*B_1) + z_P + H_v;
    end
end

if H_P < z_P + H_v
    % Piezometric head                                                      [m]
    H_P = z_P + H_v;
end

%% Calculation of flows and vapour sizes
% Volumetric flow rate                                                      [m^3/s]
Q_u_P = (C_p - H_P)/B;
% Volumetric flow rate                                                      [m^3/s]
Q_P = (H_P - C_m)/B;
% Gas cavity volume                                                         [m^3]
V_g_P = V_g_P0 + (psi*(Q_P - Q_u_P) + (1-psi)*(Q_P0 - Q_u_P0))*2*dt;

if V_g_P < 0
    % Gas cavity volume                                                     [m^3]
    V_g_P = C_3/(H_P - z_P - H_v);
end

end
