% Copyright (C) 2017, 2018 Rune Kjerrumgaard Jensen,  
% Jesper Kr Larsen, and  Kasper Lindgren Lassen
%
% Permission is hereby granted, free of charge, to any person obtaining
% a copy of this software and associated documentation files (the
% "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to
% permit persons to whom the Software is furnished to do so, subject to
% the following conditions:
%
% The above copyright notice and this permission notice shall be
% included in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL AUTHORS BE LIABLE FOR ANY
% CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
% SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Except as contained in this notice, the name of the authors shall
% not be used in advertising or otherwise to promote the sale, use or
% other dealings in this Software without prior written authorization
% from the authors.

%% Initialize matrices to reduce calculation time
% Volumetric flow rate                                                      [m^3/s]
Q_u(1:n,1:n_t) = 0;
% Volumetric flow rate                                                      [m^3/s]
Q(1:n,1:n_t) = 0;
% Piezometric head                                                          [m]
H(1:n,1:n_t) = 0;
% Time                                                                      [s]
t(1:n_t) = 0;
% Height from datum                                                         [m]
z(1:n) = 0;
% Volumetric flow rate acceleration                                         [m^3/s^2]
dQ(n,n_t-2) = 0;
% Vapour cavity volume                                                      [m^3]
V_cav(1:n,1:n_t) = 0;

%% Calculating the offset of each node from the datum (reference height)
% i indicate node number                                                    [-]
for i = 1:n
    % Height from datum                                                     [m]
    z(i) = (i-1)*dx*sind(theta);
end

%% Steady State
[Q, H] = SteadyState(Q_0, H_r, rho, D, viscosity, a, A, roughness, g,...
    dx, Q, H, n, theta, Friction_Type, f_pre);
Q_u(:,1) = Q(:,1);

%% Transient
% j indicate time step number                                               [-]
for j = 2:n_t
    % Time                                                                  [s]
    t(j) = t(j-1) + dt;
    
    %% Interior Nodes
    % The different formulaton for j = 2 and j > 2 is because Eq. 7.9
    % requires a vapour cavity volume from two time steps back. However as 
    % there is no j = -1, the steady state values (j = 1) will be used for
    % V_cav, Q, and Q_u in Eq. 7.9.
    if j == 2
        for i = 2:n-1
            [Q_u(i,j), Q(i,j), H(i,j), V_cav(i,j)] = InteriorNodes_DVCM(...
                a, g, A, rho, D, viscosity, roughness, dx, theta,...
                Q(i-1,j-1), H(i-1,j-1), Q_u(i+1,j-1), H(i+1,j-1), Q,...
                Q_u, Re_0, i, j, dt, Friction_Type, Q_0, f_pre, W, dQ,...
                n_t, V_cav(i,j-1), V_cav(i,j-1), Q(i,j-1), Q_u(i,j-1),...
                psi, z(i), H_v);
        end
    else
        for i = 2:n-1
            [Q_u(i,j), Q(i,j), H(i,j), V_cav(i,j)] = InteriorNodes_DVCM(...
                a, g, A, rho, D, viscosity, roughness, dx, theta,...
                Q(i-1,j-1), H(i-1,j-1), Q_u(i+1,j-1), H(i+1,j-1), Q,...
                Q_u, Re_0, i, j, dt, Friction_Type, Q_0, f_pre, W, dQ,...
                n_t, V_cav(i,j-1), V_cav(i,j-2), Q(i,j-2), Q_u(i,j-2),...
                psi, z(i), H_v);
        end
    end
    
    %% Upstream Boundary
    switch Upstream_boundary
        case 'Reservoir'
            [Q(1,j), H(1,j)] = Reservoir_Upstream(a, g, A, rho, D, dx,...
                viscosity, roughness, theta, Q_u(2,j-1), H(2,j-1), H_r,...
                Q_u, Re_0, 1, j, dt, Friction_Type, Q_0, f_pre, W, dQ, n_t);
    end
    Q_u(1,j) = Q(1,j);
    
    %% Downstream Boundary
    % Again, because there is no j = -1, the steady state values are used
    % for V_cav, Q, and Q_u in Eq. 7.9.
    if j == 2
        switch Downstream_boundary
            case 'Valve_Instantaneous_Closure'
                [Q_u(n,j), Q(n,j), H(n,j), V_cav(n,j)] = Valve_Closure_DVCM(...
                    a, g, A, D, dx, roughness, rho, viscosity,...
                    t(j), t_c, m, theta, Q(n,1), H(n,1), Q(n-1,j-1),...
                    H(n-1,j-1), Q, Re_0, n, j, dt, Friction_Type, f_pre,...
                    W, dQ, n_t, V_cav(n,j-1), V_cav(n,j-1), Q(n,j-1),...
                    Q_u(n,j-1), psi, z(n), H_v);
            case 'Valve_Transient_Closure'
                [Q_u(n,j), Q(n,j), H(n,j), V_cav(n,j)] = Valve_Closure_DVCM(...
                    a, g, A, D, dx, roughness, rho, viscosity,...
                    t(j), t_c, m, theta, Q(n,1), H(n,1), Q(n-1,j-1),...
                    H(n-1,j-1), Q, Re_0, n, j, dt, Friction_Type, f_pre,...
                    W, dQ, n_t, V_cav(n,j-1), V_cav(n,j-1), Q(n,j-1),...
                    Q_u(n,j-1), psi, z(n), H_v);
        end
    else
        switch Downstream_boundary
            case 'Valve_Instantaneous_Closure'
                [Q_u(n,j), Q(n,j), H(n,j), V_cav(n,j)] = Valve_Closure_DVCM(...
                    a, g, A, D, dx, roughness, rho, viscosity,...
                    t(j), t_c, m, theta, Q(n,1), H(n,1), Q(n-1,j-1),...
                    H(n-1,j-1), Q, Re_0, n, j, dt, Friction_Type, f_pre,...
                    W, dQ, n_t, V_cav(n,j-1), V_cav(n,j-2), Q(n,j-2),...
                    Q_u(n,j-2), psi, z(n), H_v);
            case 'Valve_Transient_Closure'
                [Q_u(n,j), Q(n,j), H(n,j), V_cav(n,j)] = Valve_Closure_DVCM(...
                    a, g, A, D, dx, roughness, rho, viscosity,...
                    t(j), t_c, m, theta, Q(n,1), H(n,1), Q(n-1,j-1),...
                    H(n-1,j-1), Q, Re_0, n, j, dt, Friction_Type, f_pre,...
                    W, dQ, n_t, V_cav(n,j-1), V_cav(n,j-2), Q(n,j-2),...
                    Q_u(n,j-2), psi, z(n), H_v);
        end
    end
    
    %% Calculating the change in volumetric flow rate for unsteady friction
    switch Friction_Type
        case 'Unsteady_Friction_Zielke'
            if j<n_t
            dQ(:,n_t-j+1) = Q(:,j)-Q(:,j-1);
            end
        case 'Unsteady_Friction_VardyBrown'
            if j<n_t
                dQ(:,n_t-j+1) = Q(:,j)-Q(:,j-1);
            end
        case 'Unsteady_Friction_Zarzycki'
            if j<n_t
                dQ(:,n_t-j+1) = Q(:,j)-Q(:,j-1);
            end
    end
end

%% Calculating the void fraction - used to warn for void fraction >10%
V_reach_boundary = dx/2 * A;
V_reach_interior = dx * A;

alpha(1,:) = V_cav(1,:)/V_reach_boundary;
alpha(2:n-1,:) = V_cav(2:end-1,:)/V_reach_interior;
alpha(n,:) = V_cav(end,:)/V_reach_boundary;

if max(max(alpha)) > 0.1
    fprintf(2,'Warning: A void fraction above 0.1\n')
    fprintf(2,'has been calculated, and the solution\n')
    fprintf(2,'might not be accurate. Try using \n')
    fprintf(2,'DGCM instead, as it might produce\n')
    fprintf(2,'better results.\n')
    disp('-------------------------------------')
end
